function x = profbsub(R,prof,x)
% ----------------------------------------------------------------------
% In-place backwards substitution, solving R*x = b for an upper
% triangular profile (skyline) Cholesky factor R.  R can be stored as a
% dense nxn matrix, as a band matrix or as a fully packed profile
% matrix.  In each case, prof specifies its profile shape. See
% profchol() for details. x is a dense nxk matrix, k>=1.
% ----------------------------------------------------------------------

if nargin<3, error('profbsub: missing arguments'); end
[b,n] = size(R);
nx = size(x,1);

% Skip any trailing zeros in x.
for px=nx:-1:1
   if any(x(px,:)), break; end
end

if b==n		% unpacked nxn matrix

   for i=px:-1:2
      x(i,:) = x(i,:) / R(i,i);
      J = prof(i):i-1;
      x(J,:) = x(J,:) - R(J,i)*x(i,:);
   end
   x(1,:) = x(1,:) / R(1,1);

elseif b<n		% packed band matrix

   % R(prof(i):b,i) represents column i elements A(i+prof(i)-b:i,i) of
   % unpacked matrix.

   for i=px:-1:2
      x(i,:) = x(i,:) / R(b,i);
      J = prof(i)+i-b:i-1;
      x(J,:) = x(J,:) - R(J-(i-b),i)*x(i,:);
   end
   x(1,:) = x(1,:) / R(b,1);

elseif n==1			% fully packed profile matrix

   % R(prof(i-1)+1:prof(i)) represents column i elements
   % A(prof(i-1)+1-prof(i)+i,i) of unpacked matrix. di = location of
   % A(i,i) in R, ci = nominal offset of element 0 of column i in R, pi
   % = index of first nonzero element in column i of A. 

   di = prof(px);
   for i=px:-1:2
      diprev = prof(i-1);
      ci = di-i;
      x(i,:) = x(i,:) / R(di);
      J = diprev+1-ci:i-1;
      x(J,:) = x(J,:) - R(ci+J,1)*x(i,:);
      di = diprev;
   end
   x(1,:) = x(1,:) / R(di);
   
else
   error('proffsub: unkonwn matrix format');
end
